@@ -25,34 +25,35 @@
 char *
 chxj_encoding(request_rec *r, const char *src, apr_size_t *len)
 {
-  char*               obuf;
-  char*               ibuf;
-  char*               spos;
+  char                *obuf;
+  char                *ibuf;
+  char                *spos;
   
   iconv_t             cd;
   size_t              result;
   apr_size_t          ilen;
   apr_size_t          olen;
-  mod_chxj_config*    dconf;
-  chxjconvrule_entry* entryp;
-  apr_pool_t          *pool;
+  mod_chxj_config     *dconf;
+  chxjconvrule_entry  *entryp;
 
 
   DBG(r,"start chxj_encoding()");
 
-  apr_pool_create(&pool, r->pool);
-
   dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
 
   if (dconf == NULL) {
     DBG(r,"none encoding.");
     return (char*)src;
   }
+  if ((int)*len < 0) {
+    ERR(r, "runtime exception: chxj_encoding(): invalid string size.[%d]", (int)*len);
+    return (char *)apr_pstrdup(r->pool, "");
+  }
 
   entryp = chxj_apply_convrule(r, dconf->convrules);
   if (entryp->encoding == NULL) {
     DBG(r,"none encoding.");
-    return (char*)src;
+    return (char *)src;
   }
 
   if (STRCASEEQ('n','N',"none", entryp->encoding)) {
@@ -60,16 +61,16 @@ chxj_encoding(request_rec *r, const char *src, apr_size_t *len)
     return (char*)src;
   }
   ilen = *len;
-  ibuf = apr_palloc(pool, ilen+1);
+  ibuf = apr_palloc(r->pool, ilen+1);
   if (ibuf == NULL) {
-    DBG(r,"end   chxj_encoding()");
-    return (char*)src;
+    ERR(r, "runtime exception: chxj_encoding(): Out of memory.");
+    return (char *)src;
   }
   memset(ibuf, 0, ilen+1);
   memcpy(ibuf, src, ilen);
 
   olen = ilen * 4 + 1;
-  spos = obuf = apr_palloc(pool, olen);
+  spos = obuf = apr_palloc(r->pool, olen);
   if (obuf == NULL) {
     DBG(r,"end   chxj_encoding()");
     return ibuf;
@@ -111,22 +112,27 @@ chxj_encoding(request_rec *r, const char *src, apr_size_t *len)
 }
 
 
-char*
-chxj_rencoding(request_rec *r, const char* src, apr_size_t* len)
+char *
+chxj_rencoding(request_rec *r, const char *src, apr_size_t *len)
 {
-  char*               obuf;
-  char*               ibuf;
-  char*               spos;
+  char                *obuf;
+  char                *ibuf;
+  char                *spos;
   
   iconv_t             cd;
   size_t              result;
   apr_size_t          ilen;
   apr_size_t          olen;
-  mod_chxj_config*    dconf;
-  chxjconvrule_entry* entryp;
+  mod_chxj_config     *dconf;
+  chxjconvrule_entry  *entryp;
 
   DBG(r,"start chxj_rencoding()");
 
+  if ((int)*len < 0) {
+    ERR(r, "runtime exception: chxj_rencoding(): invalid string size.[%d]", (int)*len);
+    return (char *)apr_pstrdup(r->pool, "");
+  }
+
   dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
   if (! dconf) {
     DBG(r,"none encoding.");
@@ -201,7 +207,7 @@ chxj_rencoding(request_rec *r, const char* src, apr_size_t* len)
 }
 
 
-char*
+char *
 chxj_encoding_parameter(request_rec *r, const char *value)
 {
   char *src;
@@ -233,6 +239,7 @@ chxj_encoding_parameter(request_rec *r, const char *value)
 
   for (;;) {
     apr_size_t len;
+    char *sep_pos;
 
     use_amp_flag = 0;
 
@@ -243,8 +250,24 @@ chxj_encoding_parameter(request_rec *r, const char *value)
       pair += 4;
       use_amp_flag = 1;
     }
-    key = apr_strtok(pair, "=", &vstat);
-    val = apr_strtok(NULL, "=", &vstat);
+    sep_pos = strchr(pair, '=');
+    if (pair == sep_pos) {
+      key = apr_pstrdup(r->pool, "");
+    }
+    else {
+      key = apr_strtok(pair, "=", &vstat);
+      pair = NULL;
+    }
+    if (key) {
+      key = chxj_url_decode(r->pool, key);
+      len = (apr_size_t)strlen(key);
+      key = chxj_encoding(r, key, &len);
+      key = chxj_url_encode(r->pool, key);
+    }
+    val = apr_strtok(pair, "=", &vstat);
+    if (! val && sep_pos) {
+      val = apr_pstrdup(r->pool, "");
+    }
     if (val) {
       val = chxj_url_decode(r->pool, val);
       len = (apr_size_t)strlen(val);
