@@ -30,6 +30,8 @@
 #define W_L(X)          do { jhtml->out = BUFFERED_WRITE_LITERAL(jhtml->out, &doc->buf, (X)); } while(0)
 #define W_V(X)          do { jhtml->out = (X) ? BUFFERED_WRITE_VALUE(jhtml->out, &doc->buf, (X))  \
                                               : BUFFERED_WRITE_LITERAL(jhtml->out, &doc->buf, ""); } while(0)
+#undef W_NLCODE
+#define W_NLCODE()     do { char *nlcode = TO_NLCODE(jhtml->conf); W_V(nlcode); } while (0)
 
 static char *s_jhtml_start_html_tag     (void *pdoc, Node *node);
 static char *s_jhtml_end_html_tag       (void *pdoc, Node *node);
@@ -401,8 +403,8 @@ tag_handler jhtml_handler[] = {
  * @param src   [i]   The character string before the converting is appointed.
  * @return The character string after the converting is returned.
  */
-char*
-chxj_exchange_jhtml(
+char *
+chxj_convert_jhtml(
   request_rec         *r,
   device_table        *spec,
   const char          *src,
@@ -416,7 +418,6 @@ chxj_exchange_jhtml(
   char      *ss;
   jhtml_t   jhtml;
   Doc       doc;
-  apr_pool_t *pool;
 
   dst = NULL;
 
@@ -447,8 +448,7 @@ chxj_exchange_jhtml(
   qs_init_malloc(&doc);
   qs_init_root_node(&doc);
 
-  apr_pool_create(&pool, r->pool);
-  ss = apr_pcalloc(pool, srclen + 1);
+  ss = apr_pcalloc(r->pool, srclen + 1);
 
   memset(ss,   0, srclen + 1);
   memcpy(ss, src, srclen);
@@ -463,19 +463,19 @@ chxj_exchange_jhtml(
   /*--------------------------------------------------------------------------*/
   /* It converts it from CHTML to JHTML.                                      */
   /*--------------------------------------------------------------------------*/
-  chxj_node_exchange(spec,r,(void*)&jhtml, &doc, qs_get_root(&doc), 0);
+  chxj_node_convert(spec,r,(void*)&jhtml, &doc, qs_get_root(&doc), 0);
   jhtml.out = chxj_buffered_write_flush(jhtml.out, &doc.buf);
-  dst = apr_pstrdup(pool, jhtml.out);
+  dst = apr_pstrdup(r->pool, jhtml.out);
   chxj_buffered_write_terminate(&doc.buf);
 
 
   qs_all_free(&doc,QX_LOGMARK);
 
   if (! dst) 
-    return apr_pstrdup(pool,ss);
+    return apr_pstrdup(r->pool,ss);
 
   if (! strlen(dst)) 
-    dst = apr_psprintf(pool, "\n");
+    dst = apr_psprintf(r->pool, "\n");
 
   *dstlen = strlen(dst);
 
@@ -498,7 +498,7 @@ chxj_exchange_jhtml(
  * @param spec  [i]   The pointer to the device_table
  */
 static void
-s_init_jhtml(jhtml_t* jhtml, Doc* doc, request_rec* r, device_table* spec)
+s_init_jhtml(jhtml_t *jhtml, Doc *doc, request_rec *r, device_table *spec)
 {
   memset(doc,   0, sizeof(Doc));
   memset(jhtml, 0, sizeof(jhtml_t));
@@ -506,7 +506,7 @@ s_init_jhtml(jhtml_t* jhtml, Doc* doc, request_rec* r, device_table* spec)
   doc->r      = r;
   jhtml->doc  = doc;
   jhtml->spec = spec;
-  jhtml->out  = qs_alloc_zero_byte_string(r);
+  jhtml->out  = qs_alloc_zero_byte_string(r->pool);
   jhtml->conf = chxj_get_module_config(r->per_dir_config, &chxj_module);
   jhtml->doc->parse_mode = PARSE_MODE_CHTML;
 }
@@ -605,6 +605,7 @@ s_jhtml_start_html_tag(void *pdoc, Node *UNUSED(node))
   /* start HTML tag                                                           */
   /*--------------------------------------------------------------------------*/
   W_L("<html>");
+  W_NLCODE();
 
   DBG(r, "end s_jhtml_start_html_tag()");
 
@@ -627,6 +628,7 @@ s_jhtml_end_html_tag(void *pdoc, Node *UNUSED(child))
   Doc           *doc = jhtml->doc;
 
   W_L("</html>");
+  W_NLCODE();
 
   return jhtml->out;
 }
@@ -736,6 +738,7 @@ s_jhtml_start_meta_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -777,6 +780,7 @@ s_jhtml_start_head_tag(void *pdoc, Node *UNUSED(node))
   r     = doc->r;
 
   W_L("<head>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -801,6 +805,7 @@ s_jhtml_end_head_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</head>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -849,6 +854,7 @@ s_jhtml_end_title_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</title>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -889,6 +895,7 @@ s_jhtml_start_base_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -968,16 +975,15 @@ s_jhtml_start_body_tag(void *pdoc, Node *node)
       /*----------------------------------------------------------------------*/
       /* ignore */
     }
-    else if (STRCASEEQ('v','V',"vlink",name) && value && *value) {
+    else if (STRCASEEQ('v','V',"vlink",name)) {
       /*----------------------------------------------------------------------*/
       /* CHTML 4.0                                                            */
       /*----------------------------------------------------------------------*/
-      W_L(" vlink=\"");
-      W_V(value);
-      W_L("\"");
+      /* ignore */
     }
   }
   W_L(">");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1002,6 +1008,7 @@ s_jhtml_end_body_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</body>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1150,6 +1157,7 @@ s_jhtml_end_a_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</a>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1192,6 +1200,7 @@ s_jhtml_start_br_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1231,7 +1240,8 @@ s_jhtml_start_tr_tag(void *pdoc, Node *UNUSED(node))
   doc   = jhtml->doc;
   r     = doc->r;
 
-  W_L("<br>\r\n");
+  W_L("<br>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1267,12 +1277,12 @@ s_jhtml_start_font_tag(void *pdoc, Node *node)
   Doc           *doc;
   request_rec   *r;
   Attr          *attr;
+  char          *color = NULL;
 
   jhtml = GET_JHTML(pdoc);
   doc   = jhtml->doc;
   r     = doc->r;
 
-  W_L("<font");
   /*--------------------------------------------------------------------------*/
   /* Get Attributes                                                           */
   /*--------------------------------------------------------------------------*/
@@ -1282,9 +1292,8 @@ s_jhtml_start_font_tag(void *pdoc, Node *node)
     char *name  = qs_get_attr_name(doc,attr);
     char *value = qs_get_attr_value(doc,attr);
     if (STRCASEEQ('c','C',"color",name) && value && *value) {
-      W_L(" color=\"");
-      W_V(value);
-      W_L("\"");
+      color = apr_pstrdup(doc->buf.pool, value);
+      break;
     }
     else if (STRCASEEQ('s','S',"size",name)) {
       /*----------------------------------------------------------------------*/
@@ -1293,7 +1302,12 @@ s_jhtml_start_font_tag(void *pdoc, Node *node)
       /* ignore */
     }
   }
-  W_L(">");
+  if (color) {
+    W_L("<font color=\"");
+    W_V(color);
+    W_L("\">");
+    jhtml->font_flag++;
+  }
   return jhtml->out;
 }
 
@@ -1317,7 +1331,11 @@ s_jhtml_end_font_tag(void *pdoc, Node *UNUSED(child))
   doc   = jhtml->doc;
   r     = jhtml->doc->r;
 
-  W_L("</font>");
+  if (jhtml->font_flag) {
+    W_L("</font>");
+    W_NLCODE();
+    jhtml->font_flag--;
+  }
   return jhtml->out;
 }
 
@@ -1337,15 +1355,14 @@ s_jhtml_start_form_tag(void *pdoc, Node *node)
   Doc          *doc;
   request_rec  *r;
   Attr         *attr;
-  char         *new_hidden_tag = NULL;
-  char         *attr_action = NULL;
-  char         *attr_method = NULL;
-  char         *attr_name = NULL;
+  int          dcflag = 0;
+  char         *dc = NULL;
 
   jhtml = GET_JHTML(pdoc);
   doc   = jhtml->doc;
   r     = doc->r;
 
+  W_L("<form");
   /*--------------------------------------------------------------------------*/
   /* Get Attributes                                                           */
   /*--------------------------------------------------------------------------*/
@@ -1358,13 +1375,21 @@ s_jhtml_start_form_tag(void *pdoc, Node *node)
       /*----------------------------------------------------------------------*/
       /* CHTML 1.0                                                            */
       /*----------------------------------------------------------------------*/
-      attr_action = chxj_add_cookie_parameter(r, value, jhtml->cookie);
+      W_L(" action=\"");
+      W_V(value);
+      W_L("\"");
+      dc = chxj_add_cookie_parameter(r, value, jhtml->cookie);
+      if (strcmp(dc, value)) {
+        dcflag = 1;
+      } 
     }
     else if (STRCASEEQ('m','M',"method",name)) {
       /*----------------------------------------------------------------------*/
       /* CHTML 1.0                                                            */
       /*----------------------------------------------------------------------*/
-      attr_method = apr_pstrdup(doc->pool, value);
+      W_L(" method=\"");
+      W_V(value);
+      W_L("\"");
     }
     else if (STRCASEEQ('u','U',"utn",name)) {
       /*----------------------------------------------------------------------*/
@@ -1373,43 +1398,25 @@ s_jhtml_start_form_tag(void *pdoc, Node *node)
       /*----------------------------------------------------------------------*/
       /* ignore */
     }
-    else if (STRCASEEQ('n','N',"name",name)) {
-      attr_name = apr_pstrdup(doc->pool, name);
-    }
-  }
-  int post_flag = (attr_method && strcasecmp(attr_method, "post") == 0) ? 1 : 0;
-
-  W_L("<form");
-  if (attr_action) {
-    char *q;
-    q = strchr(attr_action, '?');
-    if (q) {
-      new_hidden_tag = chxj_form_action_to_hidden_tag(r, doc->pool, attr_action, 0, post_flag);
-      if (new_hidden_tag) {
-        *q = 0;
-      }
-    }
-    W_L(" action=\"");
-    W_V(attr_action);
-    W_L("\"");
-  }
-  if (attr_method) {
-    W_L(" method=\"");
-    W_V(attr_method);
-    W_L("\"");
-  }
-  if (attr_name) {
-    W_L(" name=\"");
-    W_V(attr_name);
-    W_L("\"");
   }
   W_L(">");
   /*-------------------------------------------------------------------------*/
+  /* ``action=""''                                                           */
+  /*-------------------------------------------------------------------------*/
+  if (! dc) {
+    dcflag = 1;
+  }
+  /*-------------------------------------------------------------------------*/
   /* Add cookie parameter                                                    */
   /*-------------------------------------------------------------------------*/
-  if (new_hidden_tag) {
-    W_V(new_hidden_tag);
+  if (jhtml->cookie && jhtml->cookie->cookie_id && dcflag == 1) {
+    char *vv = apr_psprintf(doc->buf.pool, "%s<input type='hidden' name='%s' value='%s'>",
+                            jhtml->out, 
+                            CHXJ_COOKIE_PARAM,
+                            chxj_url_decode(doc->buf.pool, jhtml->cookie->cookie_id));
+    W_V(vv);
   }
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1428,6 +1435,7 @@ s_jhtml_end_form_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc     *doc   = jhtml->doc;
   W_L("</form>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1471,14 +1479,14 @@ s_jhtml_start_input_tag(void *pdoc, Node *node)
   /*--------------------------------------------------------------------------*/
   /* Get Attributes                                                           */
   /*--------------------------------------------------------------------------*/
-  type       = qs_get_type_attr(doc, node, r);
-  name       = qs_get_name_attr(doc, node, r);
-  value      = qs_get_value_attr(doc,node,r);
-  istyle     = qs_get_istyle_attr(doc,node,r);
-  max_length = qs_get_maxlength_attr(doc,node,r);
-  checked    = qs_get_checked_attr(doc,node,r);
-  accesskey  = qs_get_accesskey_attr(doc, node, r);
-  size       = qs_get_size_attr(doc, node, r);
+  type       = qs_get_type_attr(doc, node, doc->buf.pool);
+  name       = qs_get_name_attr(doc, node, doc->buf.pool);
+  value      = qs_get_value_attr(doc,node,doc->buf.pool);
+  istyle     = qs_get_istyle_attr(doc,node,doc->buf.pool);
+  max_length = qs_get_maxlength_attr(doc,node,doc->buf.pool);
+  checked    = qs_get_checked_attr(doc,node,doc->buf.pool);
+  accesskey  = qs_get_accesskey_attr(doc, node, doc->buf.pool);
+  size       = qs_get_size_attr(doc, node, doc->buf.pool);
 
   if (type) {
     if (type && (STRCASEEQ('t','T',"text",    type) ||
@@ -1505,7 +1513,7 @@ s_jhtml_start_input_tag(void *pdoc, Node *node)
   }
   if (value && *value) {
     W_L(" value=\"");
-    W_V(chxj_add_slash_to_doublequote(doc->pool,value));
+    W_V(value);
     W_L("\"");
   }
   if (accesskey && *accesskey) {
@@ -1549,6 +1557,7 @@ s_jhtml_start_input_tag(void *pdoc, Node *node)
     W_L(" checked");
   }
   W_L(">");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1583,6 +1592,7 @@ s_jhtml_start_center_tag(void *pdoc, Node *UNUSED(node))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc     *doc   = jhtml->doc;
   W_L("<center>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1607,6 +1617,7 @@ s_jhtml_end_center_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</center>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1721,6 +1732,7 @@ s_jhtml_start_ol_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1745,6 +1757,7 @@ s_jhtml_end_ol_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</ol>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1792,6 +1805,7 @@ s_jhtml_start_p_tag(void *pdoc, Node *node)
     W_L("\"");
   }
   W_L(">");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1811,6 +1825,7 @@ s_jhtml_end_p_tag(void *pdoc, Node *UNUSED(child))
   Doc       *doc   = jhtml->doc;
 
   W_L("</p>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1831,6 +1846,7 @@ s_jhtml_start_pre_tag(void *pdoc, Node *UNUSED(node))
 
   jhtml->pre_flag++;
   W_L("<pre>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1850,6 +1866,7 @@ s_jhtml_end_pre_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = jhtml->doc;
 
   W_L("</pre>");
+  W_NLCODE();
   jhtml->pre_flag--;
 
   return jhtml->out;
@@ -1871,6 +1888,7 @@ s_jhtml_start_ul_tag(void *pdoc, Node *UNUSED(node))
   Doc     *doc   = jhtml->doc;
 
   W_L("<ul>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1890,6 +1908,7 @@ s_jhtml_end_ul_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = jhtml->doc;
 
   W_L("</ul>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -1957,6 +1976,7 @@ s_jhtml_start_hr_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2042,14 +2062,21 @@ s_jhtml_start_img_tag(void *pdoc, Node *node)
       /*----------------------------------------------------------------------*/
       /* CHTML 1.0                                                            */
       /*----------------------------------------------------------------------*/
-      if (value && (STRCASEEQ('t','T',"top",   value) ||
-                    STRCASEEQ('m','M',"middle",value) ||
-                    STRCASEEQ('b','B',"bottom",value) ||
-                    STRCASEEQ('l','L',"left",  value) ||
-                    STRCASEEQ('r','R',"right", value))) {
-        W_L(" align=\"");
-        W_V(value);
-        W_L("\"");
+      if (value) {
+        if (STRCASEEQ('t','T',"top",   value) ||
+            STRCASEEQ('m','M',"middle",value) ||
+            STRCASEEQ('b','B',"bottom",value) ||
+            STRCASEEQ('l','L',"left",  value) ||
+            STRCASEEQ('r','R',"right", value)) {
+          W_L(" align=\"");
+          W_V(value);
+          W_L("\"");
+        }
+        else if (STRCASEEQ('c','C',"center",value)) {
+          W_L(" align=\"");
+          W_L("middle");
+          W_L("\"");
+        }
       }
     }
     else if (STRCASEEQ('w','W',"width",name) && value && *value) {
@@ -2090,6 +2117,7 @@ s_jhtml_start_img_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2167,6 +2195,7 @@ s_jhtml_start_select_tag(void *pdoc, Node *child)
     W_L(" multiple");
   }
   W_L(">");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2186,6 +2215,7 @@ s_jhtml_end_select_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = jhtml->doc;
 
   W_L("</select>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2322,6 +2352,7 @@ s_jhtml_end_div_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</div>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2451,6 +2482,7 @@ s_jhtml_end_textarea_tag(void *pdoc, Node *UNUSED(child))
   r     = doc->r;
 
   W_L("</textarea>");
+  W_NLCODE();
   jhtml->textarea_flag--;
 
   return jhtml->out;
@@ -2495,7 +2527,8 @@ s_jhtml_end_b_tag(void* pdoc, Node* UNUSED(child))
   jhtml_t*      jhtml = GET_JHTML(pdoc);
   Doc*          doc   = jhtml->doc;
 
-  W_L("</b>\r\n");
+  W_L("</b>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2515,18 +2548,16 @@ s_jhtml_text_tag(void* pdoc, Node* child)
   jhtml = GET_JHTML(pdoc);
   doc   = jhtml->doc;
   r     = doc->r;
-  DBG(r, "start s_jhtml_text_tag()");
 
   textval = qs_get_node_value(doc,child);
   if (strlen(textval) == 0) {
-    DBG(r, "end s_jhtml_text_tag()");
     return jhtml->out;
   }
 
   tmp = apr_palloc(r->pool, qs_get_node_size(doc,child)+1);
   memset(tmp, 0, qs_get_node_size(doc,child)+1);
 
-  tdst     = qs_alloc_zero_byte_string(r);
+  tdst     = qs_alloc_zero_byte_string(r->pool);
   memset(one_byte, 0, sizeof(one_byte));
   tdst_len = 0;
 
@@ -2564,7 +2595,6 @@ s_jhtml_text_tag(void* pdoc, Node* child)
     }
   }
   W_V(tdst);
-  DBG(r, "end s_jhtml_text_tag()");
   return jhtml->out;
 }
 
@@ -2583,6 +2613,7 @@ s_jhtml_start_blockquote_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc     *doc   = jhtml->doc;
   W_L("<blockquote>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2601,6 +2632,7 @@ s_jhtml_end_blockquote_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc     *doc   = jhtml->doc;
   W_L("</blockquote>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2619,6 +2651,7 @@ s_jhtml_start_dir_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc *doc = jhtml->doc;
   W_L("<dir>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2637,6 +2670,7 @@ s_jhtml_end_dir_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc *doc = jhtml->doc;
   W_L("</dir>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2655,6 +2689,7 @@ s_jhtml_start_dl_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc *doc = jhtml->doc;
   W_L("<dl>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2673,6 +2708,7 @@ s_jhtml_end_dl_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc *doc = jhtml->doc;
   W_L("</dl>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -2769,10 +2805,8 @@ s_jhtml_start_h1_tag(void *pdoc, Node *node)
   for (attr = qs_get_attr(doc,node);
        attr;
        attr = qs_get_next_attr(doc,attr)) {
-    char* name;
-    char* value;
-    name  = qs_get_attr_name(doc,attr);
-    value = qs_get_attr_value(doc,attr);
+    char *name  = qs_get_attr_name(doc,attr);
+    char *value = qs_get_attr_value(doc,attr);
     if (STRCASEEQ('a','A',"align", name)) {
       if (value && (STRCASEEQ('l','L',"left",value) || STRCASEEQ('r','R',"right",value) || STRCASEEQ('c','C',"center",value))) {
         jhtml->h1_align_flag++;
@@ -2812,6 +2846,7 @@ s_jhtml_end_h1_tag(void *pdoc, Node *UNUSED(child))
   if (jhtml->h1_align_flag) {
     jhtml->h1_align_flag--;
     W_L("</div>");
+    W_NLCODE();
   }
   return jhtml->out;
 }
@@ -2884,6 +2919,7 @@ s_jhtml_end_h2_tag(void *pdoc, Node *UNUSED(child))
   if (jhtml->h2_align_flag) {
     jhtml->h2_align_flag--;
     W_L("</div>");
+    W_NLCODE();
   }
   return jhtml->out;
 }
@@ -2956,6 +2992,7 @@ s_jhtml_end_h3_tag(void *pdoc, Node *UNUSED(child))
   if (jhtml->h3_align_flag) {
     jhtml->h3_align_flag--;
     W_L("</div>");
+    W_NLCODE();
   }
   return jhtml->out;
 }
@@ -2985,10 +3022,8 @@ s_jhtml_start_h4_tag(void *pdoc, Node *node)
   for (attr = qs_get_attr(doc,node);
        attr;
        attr = qs_get_next_attr(doc,attr)) {
-    char* name;
-    char* value;
-    name  = qs_get_attr_name(doc,attr);
-    value = qs_get_attr_value(doc,attr);
+    char *name  = qs_get_attr_name(doc,attr);
+    char *value = qs_get_attr_value(doc,attr);
     if (STRCASEEQ('a','A',"align", name)) {
       if (value && (STRCASEEQ('l','L',"left",value) || STRCASEEQ('r','R',"right",value) || STRCASEEQ('c','C',"center",value))) {
         jhtml->h4_align_flag++;
@@ -3017,9 +3052,9 @@ s_jhtml_start_h4_tag(void *pdoc, Node *node)
 static char *
 s_jhtml_end_h4_tag(void *pdoc, Node *UNUSED(child)) 
 {
-  jhtml_t*    jhtml;
-  Doc*          doc;
-  request_rec*  r;
+  jhtml_t      *jhtml;
+  Doc          *doc;
+  request_rec  *r;
 
   jhtml = GET_JHTML(pdoc);
   doc     = jhtml->doc;
@@ -3028,6 +3063,7 @@ s_jhtml_end_h4_tag(void *pdoc, Node *UNUSED(child))
   if (jhtml->h4_align_flag) {
     jhtml->h4_align_flag--;
     W_L("</div>");
+    W_NLCODE();
   }
   return jhtml->out;
 }
@@ -3057,10 +3093,8 @@ s_jhtml_start_h5_tag(void *pdoc, Node *node)
   for (attr = qs_get_attr(doc,node);
        attr;
        attr = qs_get_next_attr(doc,attr)) {
-    char* name;
-    char* value;
-    name  = qs_get_attr_name(doc,attr);
-    value = qs_get_attr_value(doc,attr);
+    char *name  = qs_get_attr_name(doc,attr);
+    char *value = qs_get_attr_value(doc,attr);
     if (STRCASEEQ('a','A',"align", name)) {
       if (value && (STRCASEEQ('l','L',"left",value) || STRCASEEQ('r','R',"right",value) || STRCASEEQ('c','C',"center",value))) {
         jhtml->h5_align_flag++;
@@ -3100,6 +3134,7 @@ s_jhtml_end_h5_tag(void *pdoc, Node *UNUSED(child))
   if (jhtml->h5_align_flag) {
     jhtml->h5_align_flag--;
     W_L("</div>");
+    W_NLCODE();
   }
   return jhtml->out;
 }
@@ -3129,10 +3164,8 @@ s_jhtml_start_h6_tag(void *pdoc, Node *node)
   for (attr = qs_get_attr(doc,node);
        attr;
        attr = qs_get_next_attr(doc,attr)) {
-    char* name;
-    char* value;
-    name  = qs_get_attr_name(doc,attr);
-    value = qs_get_attr_value(doc,attr);
+    char *name  = qs_get_attr_name(doc,attr);
+    char *value = qs_get_attr_value(doc,attr);
     if (STRCASEEQ('a','A',"align", name)) {
       if (value && (STRCASEEQ('l','L',"left",value) || STRCASEEQ('r','R',"right",value) || STRCASEEQ('c','C',"center",value))) {
         jhtml->h6_align_flag++;
@@ -3161,9 +3194,9 @@ s_jhtml_start_h6_tag(void *pdoc, Node *node)
 static char *
 s_jhtml_end_h6_tag(void *pdoc, Node *UNUSED(child)) 
 {
-  jhtml_t*    jhtml;
-  Doc*          doc;
-  request_rec*  r;
+  jhtml_t     *jhtml;
+  Doc         *doc;
+  request_rec *r;
 
   jhtml = GET_JHTML(pdoc);
   doc     = jhtml->doc;
@@ -3172,6 +3205,7 @@ s_jhtml_end_h6_tag(void *pdoc, Node *UNUSED(child))
   if (jhtml->h6_align_flag) {
     jhtml->h6_align_flag--;
     W_L("</div>");
+    W_NLCODE();
   }
   return jhtml->out;
 }
@@ -3191,6 +3225,7 @@ s_jhtml_start_menu_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc     *doc = jhtml->doc;
   W_L("<menu>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -3209,6 +3244,7 @@ s_jhtml_end_menu_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc     *doc = jhtml->doc;
   W_L("</menu>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -3300,6 +3336,7 @@ s_jhtml_end_blink_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc     *doc = jhtml->doc;
   W_L("</blink>");
+  W_NLCODE();
   return jhtml->out;
 }
 
@@ -3360,6 +3397,7 @@ s_jhtml_end_marquee_tag(void *pdoc, Node *UNUSED(child))
   jhtml_t *jhtml = GET_JHTML(pdoc);
   Doc *doc = jhtml->doc;
   W_L("</marquee>");
+  W_NLCODE();
   return jhtml->out;
 }
 /*
