@@ -17,17 +17,22 @@
 #include "chxj_str_util.h"
 
 int
-chxj_chk_numeric(const char* s)
+chxj_chk_numeric(const char *s)
 {
-  int len = strlen(s);
+  int len;
   int ii;
 
+  if (! s) return -1; /* NG(not numeric) */
+
+  len = strlen(s);
+  if (len == 0) return -1; /* NG(not numeric) */
+
   for (ii=0; ii<len; ii++) {
     if (ii == 0 && (s[ii] < '0' || s[ii] > '9') && s[ii] != '-') {
-      return -1; /* NG */
+      return -1; /* NG(not numeric) */
     }
     if (ii != 0 && (s[ii] < '0' || s[ii] > '9')) {
-      return -1; /* NG */
+      return -1; /* NG(not numeric) */
     }
   }
 
@@ -37,16 +42,21 @@ chxj_chk_numeric(const char* s)
 
 
 int
-chxj_atoi(const char* s)
+chxj_atoi(const char *s)
 {
-  int len = strlen(s);
+  int len;
   int ii;
   int result;
   int mflag = 0;
+  int break_flag = 0;
+
+  if (! s) return 0;
 
   result = 0;
 
-  for (ii=0; ii < len; ii++) {
+  len = strlen(s);
+
+  for (ii=0; ii < len && break_flag == 0; ii++) {
     result *= 10;
     switch(s[ii]) {
     case '0': result += 0; break; 
@@ -60,12 +70,26 @@ chxj_atoi(const char* s)
     case '8': result += 8; break;
     case '9': result += 9; break;
     case '-': 
-      if (ii == 0)
-        mflag = 1;
+      result /= 10;
+      if (ii != 0) {
+        break_flag = 1;
+        break;
+      }
+      mflag = 1;
       break;
 
-    default: break;
+    case '+':
+      result /= 10;
+      if (ii != 0) {
+        break_flag = 1;
+        break;
+      }
+      break;
 
+    default: 
+      break_flag = 1;
+      result /= 10;
+      break;
     }
   }
 
@@ -85,8 +109,18 @@ chxj_strcasenrcmp(apr_pool_t *p, const char *s1, const char *s2, int n)
     register char *ss1p;
     register char *ss2p;
 
-    s1_len = strlen(s1) - 1;
-    s2_len = strlen(s2) - 1;
+    if (! s1) {
+      s1_len = 0;
+    }
+    else {
+      s1_len = strlen(s1) - 1;
+    }
+    if (! s2) {
+      s2_len = 0;
+    }
+    else {
+      s2_len = strlen(s2) - 1;
+    }
 
     ss1 = (char *)apr_palloc(p, s1_len + 2);
     if (!ss1) {
@@ -97,15 +131,18 @@ chxj_strcasenrcmp(apr_pool_t *p, const char *s1, const char *s2, int n)
       return -1;
     }
 
-    strcpy(&ss1[1], s1);
-    strcpy(&ss2[1], s2);
-    ss1[0] = 0;
-    ss2[0] = 0;
+    ss1[1] = ss1[0] = 0;
+    ss2[1] = ss2[0] = 0;
+    if (s1) {
+      strcpy(&ss1[1], s1);
+    }
+    if (s2) {
+      strcpy(&ss2[1], s2);
+    }
     ss1p = &ss1[s1_len+1];
     ss2p = &ss2[s2_len+1];
 
     for (;*ss1p && *ss2p && *ss1p == *ss2p && n - 1 > 0; ss1p--, ss2p--, n--);
-
     return (int)(*ss1p - *ss2p);
 }
 
@@ -113,41 +150,14 @@ chxj_strcasenrcmp(apr_pool_t *p, const char *s1, const char *s2, int n)
 int
 chxj_starts_with(const char *str, const char *word)
 {
-  int len = strlen(word);
-  return strncasecmp(str, word, len) == 0;
-}
-
-char *
-chxj_add_slash_to_doublequote(apr_pool_t *pool, const char *str)
-{
-  char *ret;
-  int  len;
-  int  tlen;
-  int  ii;
-  int  pos;
-  int  cnt;
-
-  len = strlen(str);
-  cnt = 0;
-  for (ii=0; ii<len; ii++) {
-    if (str[ii] == '"') {
-      cnt++;
-    }
-  }
-  tlen = (len - cnt)  + (cnt * (sizeof("&quot;")-1)) + 1;
-  ret = apr_palloc(pool, tlen);
-  memset(ret, 0, tlen);
-  pos = 0;
-  for (ii=0; ii<len; ii++) {
-    if (str[ii] == '"') {
-      strcpy(&ret[pos], "&quot;");
-      pos += sizeof("&quot;")-1;
-    }
-    else {
-      ret[pos++] = str[ii];
-    }
-  }
-  return ret;
+  int len;
+  char *w = (char *)word;
+  char *s = (char *)str;
+  if (! w) w = "";
+  if (! s) s = "";
+  len = strlen(w);
+  if (len == 0) len = 1;
+  return strncasecmp(s, w, len) == 0;
 }
 /*
  * vim:ts=2 et
