@@ -30,6 +30,8 @@
 #define W_L(X)          do { xhtml->out = BUFFERED_WRITE_LITERAL(xhtml->out, &doc->buf, (X)); } while(0)
 #define W_V(X)          do { xhtml->out = (X) ? BUFFERED_WRITE_VALUE(xhtml->out, &doc->buf, (X))  \
                                                : BUFFERED_WRITE_LITERAL(xhtml->out, &doc->buf, ""); } while(0)
+#undef W_NLCODE
+#define W_NLCODE()     do { char *nlcode = TO_NLCODE(xhtml->conf); W_V(nlcode); } while (0)
 
 static char *s_xhtml_1_0_start_html_tag   (void *pdoc, Node *node);
 static char *s_xhtml_1_0_end_html_tag     (void *pdoc, Node *node);
@@ -397,14 +399,14 @@ tag_handler xhtml_handler[] = {
  * @return The character string after the converting is returned.
  */
 char *
-chxj_exchange_xhtml_mobile_1_0(
+chxj_convert_xhtml_mobile_1_0(
   request_rec        *r,
   device_table       *spec,
   const char         *src,
   apr_size_t         srclen,
   apr_size_t         *dstlen,
   chxjconvrule_entry *entryp,
-  cookie_t           *cookie
+  cookie_t           *UNUSED(cookie)
 )
 {
   char      *dst = NULL;
@@ -412,14 +414,14 @@ chxj_exchange_xhtml_mobile_1_0(
   xhtml_t   xhtml;
   Doc       doc;
 
-  DBG(r,"start chxj_exchange_xhtml_mobile_1_0()");
+  DBG(r,"start chxj_convert_xhtml_mobile_1_0()");
   /*--------------------------------------------------------------------------*/
   /* If qrcode xml                                                            */
   /*--------------------------------------------------------------------------*/
   *dstlen = srclen;
   dst = chxj_qr_code_blob_handler(r, src, (size_t*)dstlen);
   if (dst != NULL) {
-    DBG(r,"end chxj_exchange_xhtml_mobile_1_0() (found qrcode.xml)");
+    DBG(r,"end chxj_convert_xhtml_mobile_1_0() (found qrcode.xml)");
     return dst;
   }
 
@@ -429,7 +431,6 @@ chxj_exchange_xhtml_mobile_1_0(
   s_init_xhtml(&xhtml, &doc, r, spec);
 
   xhtml.entryp = entryp;
-  xhtml.cookie = cookie;
 
   chxj_set_content_type(r, "text/html; charset=Windows-31J");
 
@@ -452,7 +453,7 @@ chxj_exchange_xhtml_mobile_1_0(
   /*--------------------------------------------------------------------------*/
   /* It converts it from CHTML to XHTML.                                      */
   /*--------------------------------------------------------------------------*/
-  chxj_node_exchange(spec,r,(void*)&xhtml, &doc, qs_get_root(&doc), 0);
+  chxj_node_convert(spec,r,(void *)&xhtml, &doc, qs_get_root(&doc), 0);
   xhtml.out = chxj_buffered_write_flush(xhtml.out, &doc.buf);
   dst = apr_pstrdup(r->pool, xhtml.out);
   chxj_buffered_write_terminate(&doc.buf);
@@ -472,7 +473,7 @@ chxj_exchange_xhtml_mobile_1_0(
   chxj_dump_out("[dst] CHTML->XHTML", dst, *dstlen);
 #endif
 
-  DBG(r,"end chxj_exchange_xhtml_mobile_1_0()");
+  DBG(r,"end chxj_convert_xhtml_mobile_1_0()");
   return dst;
 }
 
@@ -496,7 +497,7 @@ s_init_xhtml(xhtml_t *xhtml, Doc *doc, request_rec *r, device_table *spec)
   doc->r      = r;
   xhtml->doc  = doc;
   xhtml->spec = spec;
-  xhtml->out  = qs_alloc_zero_byte_string(r);
+  xhtml->out  = qs_alloc_zero_byte_string(r->pool);
   xhtml->conf = chxj_get_module_config(r->per_dir_config, &chxj_module);
   xhtml->doc->parse_mode = PARSE_MODE_CHTML;
 }
@@ -655,12 +656,15 @@ s_xhtml_1_0_start_html_tag(void *pdoc, Node *node)
   /*--------------------------------------------------------------------------*/
   /* Add XML Declare                                                          */
   /*--------------------------------------------------------------------------*/
-  W_L("<?xml version=\"1.0\" encoding=\"Windows-31J\"?>\r\n");
+  W_L("<?xml version=\"1.0\" encoding=\"Windows-31J\"?>");
+  W_NLCODE();
   /*--------------------------------------------------------------------------*/
   /* Add DocType                                                              */
   /*--------------------------------------------------------------------------*/
-  W_L("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML Basic 1.0//EN\"\r\n");
-  W_L(" \"http://www.w3.org/TR/xhtml-basic/xhtml-basic10.dtd\">\r\n");
+  W_L("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML Basic 1.0//EN\"");
+  W_NLCODE();
+  W_L(" \"http://www.w3.org/TR/xhtml-basic/xhtml-basic10.dtd\">");
+  W_NLCODE();
   /*--------------------------------------------------------------------------*/
   /* start HTML tag                                                           */
   /*--------------------------------------------------------------------------*/
@@ -682,7 +686,8 @@ s_xhtml_1_0_start_html_tag(void *pdoc, Node *node)
       W_L(" version=\"-//OPENWAVE//DTD XHTML Mobile 1.0//EN\"");
     }
   }
-  W_L(">\r\n");
+  W_L(">");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -700,7 +705,8 @@ s_xhtml_1_0_end_html_tag(void *pdoc, Node *UNUSED(child))
 {
   xhtml_t       *xhtml = GET_XHTML(pdoc);
   Doc           *doc   = xhtml->doc;
-  W_L("</html>\r\n");
+  W_L("</html>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -764,7 +770,8 @@ s_xhtml_1_0_start_meta_tag(void *pdoc, Node *node)
       }
     }
   }
-  W_L(" />\r\n");
+  W_L(" />");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -800,7 +807,8 @@ s_xhtml_1_0_start_head_tag(void *pdoc, Node *UNUSED(node))
   xhtml_t       *xhtml = GET_XHTML(pdoc);
   Doc           *doc   = xhtml->doc;
 
-  W_L("<head>\r\n");
+  W_L("<head>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -819,7 +827,8 @@ s_xhtml_1_0_end_head_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t       *xhtml = GET_XHTML(pdoc);
   Doc           *doc   = xhtml->doc;
 
-  W_L("</head>\r\n");
+  W_L("</head>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -857,7 +866,9 @@ s_xhtml_1_0_end_title_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t       *xhtml = GET_XHTML(pdoc);
   Doc           *doc   = xhtml->doc;
 
-  W_L("</title>\r\n");
+  W_L("</title>");
+  W_NLCODE();
+
   return xhtml->out;
 }
 
@@ -893,7 +904,9 @@ s_xhtml_1_0_start_base_tag(void *pdoc, Node *node)
       break;
     }
   }
-  W_L(" />\r\n");
+  W_L(" />");
+  W_NLCODE();
+
   return xhtml->out;
 }
 
@@ -961,7 +974,8 @@ s_xhtml_1_0_start_body_tag(void *pdoc, Node *node)
       /* ignore */
     }
   }
-  W_L(">\r\n");
+  W_L(">");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -980,7 +994,8 @@ s_xhtml_1_0_end_body_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t       *xhtml = GET_XHTML(pdoc);
   Doc           *doc   = xhtml->doc;
 
-  W_L("</body>\r\n");
+  W_L("</body>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -1017,7 +1032,6 @@ s_xhtml_1_0_start_a_tag(void *pdoc, Node *node)
     }
     else if (STRCASEEQ('h','H',"href", name) && value && *value) {
       value = chxj_encoding_parameter(r, value);
-      value = chxj_add_cookie_parameter(r, value, xhtml->cookie);
       W_L(" href=\"");
       W_V(value);
       W_L("\"");
@@ -1078,6 +1092,8 @@ s_xhtml_1_0_end_a_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = xhtml->doc;
 
   W_L("</a>");
+  W_NLCODE();
+
   return xhtml->out;
 }
 
@@ -1114,7 +1130,9 @@ s_xhtml_1_0_start_br_tag(void *pdoc, Node *node)
       }
     }
   }
-  W_L(" />\r\n");
+  W_L(" />");
+  W_NLCODE();
+
   return xhtml->out;
 }
 
@@ -1150,7 +1168,9 @@ s_xhtml_1_0_start_tr_tag(void *pdoc, Node *UNUSED(node))
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc   = xhtml->doc;
 
-  W_L("<br />\r\n");
+  W_L("<br />");
+  W_NLCODE();
+
   return xhtml->out;
 }
 
@@ -1186,8 +1206,9 @@ s_xhtml_1_0_start_font_tag(void *pdoc, Node *node)
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc   = xhtml->doc;
   Attr    *attr;
+  char    *size = NULL;
+  char    *color = NULL;
 
-  W_L("<font");
   /*=========================================================================*/
   /* Get Attributes                                                          */
   /*=========================================================================*/
@@ -1197,17 +1218,65 @@ s_xhtml_1_0_start_font_tag(void *pdoc, Node *node)
     char *name = qs_get_attr_name(doc,attr);
     char *value = qs_get_attr_value(doc,attr);
     if (STRCASEEQ('c','C',"color",name) && value && *value) {
-      W_L(" color=\"");
-      W_V(value);
-      W_L("\"");
+      color = apr_pstrdup(doc->buf.pool, value);
     }
     else if (STRCASEEQ('s','S',"size",name) && value && *value) {
-      W_L(" size=\"");
-      W_V(value);
-      W_L("\"");
+      size = apr_pstrdup(doc->buf.pool, value);
+    }
+  }
+  if (color) {
+    W_L("<font color=\"");
+    W_V(color);
+    W_L("\">");
+    xhtml->font_color_flag++;
+  }
+  if (size) {
+    xhtml->font_size_flag++;
+    switch(*size) {
+    case '1': W_L("<span style=\"font-size: xx-small\">"); break;
+    case '2': W_L("<span style=\"font-size: x-small\">");  break;
+    case '3': W_L("<span style=\"font-size: small\">");    break;
+    case '4': W_L("<span style=\"font-size: medium\">");   break;
+    case '5': W_L("<span style=\"font-size: large\">");    break;
+    case '6': W_L("<span style=\"font-size: x-large\">");  break;
+    case '7': W_L("<span style=\"font-size: xx-large\">"); break;
+    case '-':
+      if (*(size + 1) == '1') {
+        W_L("<span style=\"font-size: small\">");
+        break;
+      }
+      if (*(size + 1) == '2') {
+        W_L("<span style=\"font-size: x-small\">");
+        break;
+      }
+      if (*(size + 1) == '3') {
+        W_L("<span style=\"font-size: xx-small\">");
+        break;
+      }
+      xhtml->font_size_flag--;
+      break;
+
+    case '+':
+      if (*(size + 1) == '1') {
+        W_L("<span style=\"font-size: large\">");
+        break;
+      }
+      if (*(size + 1) == '2') {
+        W_L("<span style=\"font-size: x-large\">");
+        break;
+      }
+      if (*(size + 1) == '3') {
+        W_L("<span style=\"font-size: xx-large\">");
+        break;
+      }
+      xhtml->font_size_flag--;
+      break;
+
+    default:
+      WRN(doc->r, "invlalid font size. [%s] != (1|2|3|4|5|6|7|+1|+2|+3|-1|-2|-3)", size);
+      xhtml->font_size_flag--;
     }
   }
-  W_L(">");
   return xhtml->out;
 }
 
@@ -1226,7 +1295,15 @@ s_xhtml_1_0_end_font_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc   = xhtml->doc;
 
-  W_L("</font>");
+  if (xhtml->font_size_flag) {
+    W_L("</span>");
+    xhtml->font_size_flag--;
+  }
+  if (xhtml->font_color_flag) {
+    W_L("</font>");
+    W_NLCODE();
+    xhtml->font_color_flag--;
+  }
   return xhtml->out;
 }
 
@@ -1246,11 +1323,8 @@ s_xhtml_1_0_start_form_tag(void *pdoc, Node *node)
   Doc         *doc   = xhtml->doc;
   request_rec *r     = doc->r;
   Attr        *attr;
-  char        *new_hidden_tag = NULL;
-  char        *attr_action = NULL;
-  char        *attr_method = NULL;
-  char        *attr_name = NULL;
 
+  W_L("<form");
   /*--------------------------------------------------------------------------*/
   /* Get Attributes                                                           */
   /*--------------------------------------------------------------------------*/
@@ -1260,50 +1334,22 @@ s_xhtml_1_0_start_form_tag(void *pdoc, Node *node)
     char *name = qs_get_attr_name(doc,attr);
     char *value = qs_get_attr_value(doc,attr);
     if (STRCASEEQ('a','A',"action",name)) {
-      attr_action = chxj_encoding_parameter(r, value);
-      attr_action = chxj_add_cookie_parameter(r, attr_action, xhtml->cookie);
+      value = chxj_encoding_parameter(r, value);
+      W_L(" action=\"");
+      W_V(value);
+      W_L("\"");
     }
     else if (STRCASEEQ('m','M',"method",name)) {
-      attr_method = apr_pstrdup(doc->pool, value);
+      W_L(" method=\"");
+      W_V(value);
+      W_L("\"");
     }
     else if (STRCASEEQ('u','U',"utn",name)) {
       /* ignore */
     }
-    else if (STRCASEEQ('n','N',"name",name)) {
-      attr_name = apr_pstrdup(doc->pool, value);
-    }
-  }
-
-  int post_flag = (attr_method && strcasecmp(attr_method, "post") == 0) ? 1 : 0;
-
-  W_L("<form");
-  if (attr_action) {
-    char *q;
-    q = strchr(attr_action, '?');
-    if (q) {
-      new_hidden_tag = chxj_form_action_to_hidden_tag(r, doc->pool, attr_action, 1, post_flag);
-      if (new_hidden_tag) {
-        *q = 0;
-      }
-    }
-    W_L(" action=\"");
-    W_V(attr_action);
-    W_L("\"");
-  }
-  if (attr_method) {
-    W_L(" method=\"");
-    W_V(attr_method);
-    W_L("\"");
-  }
-  if (attr_name) {
-    W_L(" name=\"");
-    W_V(attr_name);
-    W_L("\"");
   }
   W_L(">");
-  if (new_hidden_tag) {
-    W_V(new_hidden_tag);
-  }
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -1323,6 +1369,8 @@ s_xhtml_1_0_end_form_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = xhtml->doc;
 
   W_L("</form>");
+  W_NLCODE();
+
   return xhtml->out;
 }
 
@@ -1354,14 +1402,14 @@ s_xhtml_1_0_start_input_tag(void *pdoc, Node *node)
   /*--------------------------------------------------------------------------*/
   /* Get Attributes                                                           */
   /*--------------------------------------------------------------------------*/
-  type       = qs_get_type_attr(doc, node, r);
-  name       = qs_get_name_attr(doc, node, r);
-  value      = qs_get_value_attr(doc,node,r);
-  istyle     = qs_get_istyle_attr(doc,node,r);
-  max_length = qs_get_maxlength_attr(doc,node,r);
-  checked    = qs_get_checked_attr(doc,node,r);
-  accesskey  = qs_get_accesskey_attr(doc, node, r);
-  size       = qs_get_size_attr(doc, node, r);
+  type       = qs_get_type_attr(doc, node, doc->buf.pool);
+  name       = qs_get_name_attr(doc, node, doc->buf.pool);
+  value      = qs_get_value_attr(doc,node, doc->buf.pool);
+  istyle     = qs_get_istyle_attr(doc,node,doc->buf.pool);
+  max_length = qs_get_maxlength_attr(doc,node,doc->buf.pool);
+  checked    = qs_get_checked_attr(doc,node, doc->buf.pool);
+  accesskey  = qs_get_accesskey_attr(doc, node, doc->buf.pool);
+  size       = qs_get_size_attr(doc, node, doc->buf.pool);
 
   if (type) {
     type = qs_trim_string(doc->buf.pool, type);
@@ -1389,7 +1437,7 @@ s_xhtml_1_0_start_input_tag(void *pdoc, Node *node)
   }
   if (value && *value) {
     W_L(" value=\"");
-    W_V(chxj_add_slash_to_doublequote(doc->pool,value));
+    W_V(value);
     W_L("\"");
   }
   if (accesskey && *accesskey) {
@@ -1448,6 +1496,8 @@ s_xhtml_1_0_start_input_tag(void *pdoc, Node *node)
     W_L(" checked=\"checked\"");
   }
   W_L(" />");
+  W_NLCODE();
+
   return xhtml->out;
 }
 
@@ -1484,6 +1534,7 @@ s_xhtml_1_0_start_center_tag(void *pdoc, Node *UNUSED(node))
   Doc     *doc   = xhtml->doc;
 
   W_L("<center>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -1503,6 +1554,7 @@ s_xhtml_1_0_end_center_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = xhtml->doc;
 
   W_L("</center>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -1565,14 +1617,17 @@ s_xhtml_1_0_start_hr_tag(void *pdoc, Node *node)
       /*----------------------------------------------------------------------*/
       /* ignore */
     }
-    else if (STRCASEEQ('c','C',"color", name)) {
+    else if (STRCASEEQ('c','C',"color", name) && value && *value) {
       /*----------------------------------------------------------------------*/
       /* CHTML 4.0                                                            */
       /*----------------------------------------------------------------------*/
-      /* ignore */
+      W_L(" color=\"");
+      W_V(value);
+      W_L("\"");
     }
   }
   W_L(" />");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -1610,6 +1665,7 @@ s_xhtml_1_0_start_pre_tag(void* pdoc, Node* UNUSED(node))
 
   xhtml->pre_flag++;
   W_L("<pre>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -1629,6 +1685,7 @@ s_xhtml_1_0_end_pre_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = xhtml->doc;
 
   W_L("</pre>");
+  W_NLCODE();
   xhtml->pre_flag--;
 
   return xhtml->out;
@@ -1673,6 +1730,7 @@ s_xhtml_1_0_start_p_tag(void *pdoc, Node *node)
     W_L("\"");
   }
   W_L(">");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -1692,6 +1750,7 @@ s_xhtml_1_0_end_p_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = xhtml->doc;
 
   W_L("</p>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -1705,12 +1764,30 @@ s_xhtml_1_0_end_p_tag(void *pdoc, Node *UNUSED(child))
  * @return The conversion result is returned.
  */
 static char *
-s_xhtml_1_0_start_ul_tag(void *pdoc, Node *UNUSED(node)) 
+s_xhtml_1_0_start_ul_tag(void *pdoc, Node *node)
 {
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc   = xhtml->doc;
-
-  W_L("<ul>");
+  Attr    *attr;
+  W_L("<ul");
+  /*--------------------------------------------------------------------------*/
+  /* Get Attributes                                                           */
+  /*--------------------------------------------------------------------------*/
+  for (attr = qs_get_attr(doc,node);
+       attr;
+       attr = qs_get_next_attr(doc,attr)) {
+    char *name   = qs_get_attr_name(doc,attr);
+    char *value  = qs_get_attr_value(doc,attr);
+    if (STRCASEEQ('t','T',"type",name)) {
+      if (value && (STRCASEEQ('d','D',"disc",value) || STRCASEEQ('c','C',"circle",value) || STRCASEEQ('s','S',"square",value))) {
+        W_L(" type=\"");
+        W_V(value);
+        W_L("\"");
+      }
+    }
+  }
+  W_L(">");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -1730,6 +1807,7 @@ s_xhtml_1_0_end_ul_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = xhtml->doc;
 
   W_L("</ul>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -1798,6 +1876,7 @@ s_xhtml_1_0_end_h1_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = xhtml->doc;
 
   W_L("</h1>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -1866,6 +1945,7 @@ s_xhtml_1_0_end_h2_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = xhtml->doc;
 
   W_L("</h2>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -1934,6 +2014,7 @@ s_xhtml_1_0_end_h3_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = xhtml->doc;
 
   W_L("</h3>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2002,6 +2083,7 @@ s_xhtml_1_0_end_h4_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = xhtml->doc;
 
   W_L("</h4>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2070,6 +2152,7 @@ s_xhtml_1_0_end_h5_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = xhtml->doc;
 
   W_L("</h5>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2138,6 +2221,7 @@ s_xhtml_1_0_end_h6_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = xhtml->doc;
 
   W_L("</h6>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2178,6 +2262,7 @@ s_xhtml_1_0_start_ol_tag(void *pdoc, Node *node)
     }
   }
   W_L(">");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2197,6 +2282,7 @@ s_xhtml_1_0_end_ol_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = xhtml->doc;
 
   W_L("</ol>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2223,12 +2309,14 @@ s_xhtml_1_0_start_li_tag(void *pdoc, Node *node)
   for (attr = qs_get_attr(doc,node);
        attr;
        attr = qs_get_next_attr(doc,attr)) {
-    char *name = qs_get_attr_name(doc,attr);
+    char *name  = qs_get_attr_name(doc,attr);
     char *value = qs_get_attr_value(doc,attr);
-    if (STRCASEEQ('t','T',"type",name) && value && (*value == '1' || *value == 'a' || *value == 'A')) {
-      W_L(" type=\"");
-      W_V(value);
-      W_L("\"");
+    if (STRCASEEQ('t','T',"type",name)) {
+      if (value && (*value == '1' || *value == 'a' || *value == 'A' || STRCASEEQ('d','D',"disc",value) || STRCASEEQ('c','C',"circle",value) || STRCASEEQ('s','S',"square",value))) {
+        W_L(" type=\"");
+        W_V(value);
+        W_L("\"");
+      }
     }
     else if (STRCASEEQ('v','V',"value", name) && value && *value) {
       W_L(" value=\"");
@@ -2256,6 +2344,7 @@ s_xhtml_1_0_end_li_tag(void *pdoc, Node *UNUSED(child))
   Doc     *doc   = xhtml->doc;
 
   W_L("</li>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2291,14 +2380,6 @@ s_xhtml_1_0_start_img_tag(void *pdoc, Node *node)
 
     if (STRCASEEQ('s','S',"src",name)) {
       value = chxj_encoding_parameter(r, value);
-      value = chxj_add_cookie_parameter(r, value, xhtml->cookie);
-      if (value) {
-        value = apr_psprintf(r->pool,
-                             "%s%c%s=true",
-                             value,
-                             (strchr(value, '?')) ? '&' : '?',
-                             CHXJ_COOKIE_NOUPDATE_PARAM);
-      }
 #ifdef IMG_NOT_CONVERT_FILENAME
 
       W_L(" src=\"");
@@ -2318,14 +2399,21 @@ s_xhtml_1_0_start_img_tag(void *pdoc, Node *node)
     }
     else 
     if (STRCASEEQ('a','A',"align",name)) {
-      if (value && (STRCASEEQ('t','T',"top",   value) ||
-                    STRCASEEQ('m','M',"middle",value) ||
-                    STRCASEEQ('b','B',"bottom",value) ||
-                    STRCASEEQ('l','L',"left",  value) ||
-                    STRCASEEQ('r','R',"right", value))) {
-        W_L(" align=\"");
-        W_V(value);
-        W_L("\"");
+      if (value) {
+        if (STRCASEEQ('t','T',"top",   value) ||
+            STRCASEEQ('m','M',"middle",value) ||
+            STRCASEEQ('b','B',"bottom",value) ||
+            STRCASEEQ('l','L',"left",  value) ||
+            STRCASEEQ('r','R',"right", value)) {
+          W_L(" align=\"");
+          W_V(value);
+          W_L("\"");
+        }
+        else if (STRCASEEQ('c','C',"center",  value)) {
+          W_L(" align=\"");
+          W_L("middle");
+          W_L("\"");
+        }
       }
     }
     else if (STRCASEEQ('a','A',"alt",name) && value && *value) {
@@ -2355,6 +2443,7 @@ s_xhtml_1_0_start_img_tag(void *pdoc, Node *node)
     }
   }
   W_L(" />");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2434,7 +2523,8 @@ s_xhtml_1_0_start_select_tag(void *pdoc, Node *child)
     /* "true" is *NOT* W3C. it is specification of WAP2.0 for EZWEB */
     W_L(" multiple=\"true\"");
   }
-  W_L(">\r\n");
+  W_L(">");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2453,7 +2543,8 @@ s_xhtml_1_0_end_select_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc   = xhtml->doc;
 
-  W_L("</select>\r\n");
+  W_L("</select>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2518,7 +2609,8 @@ s_xhtml_1_0_end_option_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc   = xhtml->doc;
 
-  W_L("</option>\r\n");
+  W_L("</option>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2560,6 +2652,7 @@ s_xhtml_1_0_start_div_tag(void *pdoc, Node *child)
     W_L("\"");
   }
   W_L(">");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2577,7 +2670,8 @@ s_xhtml_1_0_end_div_tag(void *pdoc, Node *UNUSED(child))
 {
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc   = xhtml->doc;
-  W_L("</div>\r\n");
+  W_L("</div>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2615,7 +2709,8 @@ s_xhtml_1_0_end_b_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc   = xhtml->doc;
 
-  W_L("</div>\n");
+  W_L("</div>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2694,7 +2789,7 @@ s_xhtml_1_0_start_textarea_tag(void *pdoc, Node *node)
       W_L("\"");
     }
   }
-  W_L(">\r\n");
+  W_L(">");
   return xhtml->out;
 }
 
@@ -2713,7 +2808,8 @@ s_xhtml_1_0_end_textarea_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc   = xhtml->doc;
 
-  W_L("</textarea>\r\n");
+  W_L("</textarea>");
+  W_NLCODE();
   xhtml->textarea_flag--;
 
   return xhtml->out;
@@ -2740,7 +2836,7 @@ s_xhtml_1_0_text_tag(void *pdoc, Node *child)
   tmp = apr_palloc(r->pool, qs_get_node_size(doc,child)+1);
   memset(tmp, 0, qs_get_node_size(doc,child)+1);
   
-  tdst     = qs_alloc_zero_byte_string(r);
+  tdst     = qs_alloc_zero_byte_string(doc->buf.pool);
   memset(one_byte, 0, sizeof(one_byte));
   tdst_len = 0;
   
@@ -2792,6 +2888,7 @@ s_xhtml_1_0_start_blockquote_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc   = xhtml->doc;
   W_L("<blockquote>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2810,6 +2907,7 @@ s_xhtml_1_0_end_blockquote_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc   = xhtml->doc;
   W_L("</blockquote>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2823,11 +2921,27 @@ s_xhtml_1_0_end_blockquote_tag(void *pdoc, Node *UNUSED(child))
  * @return The conversion result is returned.
  */
 static char *
-s_xhtml_1_0_start_dir_tag(void *pdoc, Node *UNUSED(child))
+s_xhtml_1_0_start_dir_tag(void *pdoc, Node *node)
 {
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc = xhtml->doc;
-  W_L("<dir>");
+  Attr      *attr;
+  W_L("<dir");
+  for (attr = qs_get_attr(doc,node);
+       attr;
+       attr = qs_get_next_attr(doc,attr)) {
+    char *name   = qs_get_attr_name(doc,attr);
+    char *value  = qs_get_attr_value(doc,attr);
+    if (STRCASEEQ('t','T',"type",name)) {
+      if (value && (STRCASEEQ('d','D',"disc",value) || STRCASEEQ('c','C',"circle",value) || STRCASEEQ('s','S',"square",value))) {
+        W_L(" type=\"");
+        W_V(value);
+        W_L("\"");
+      }
+    }
+  }
+  W_L(">");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2846,6 +2960,7 @@ s_xhtml_1_0_end_dir_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc = xhtml->doc;
   W_L("</dir>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2864,6 +2979,7 @@ s_xhtml_1_0_start_dl_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc *doc = xhtml->doc;
   W_L("<dl>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2882,6 +2998,7 @@ s_xhtml_1_0_end_dl_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc *doc = xhtml->doc;
   W_L("</dl>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2967,13 +3084,30 @@ s_xhtml_1_0_end_dd_tag(void *pdoc, Node *UNUSED(child))
  * @return The conversion result is returned.
  */
 static char *
-s_xhtml_1_0_start_menu_tag(void *pdoc, Node *UNUSED(child))
+s_xhtml_1_0_start_menu_tag(void *pdoc, Node *node)
 {
-  xhtml_t *xhtml;
-  Doc *doc;
-  xhtml = GET_XHTML(pdoc);
-  doc     = xhtml->doc;
-  W_L("<menu>");
+  xhtml_t *xhtml = GET_XHTML(pdoc);
+  Doc     *doc   = xhtml->doc;
+  Attr    *attr;
+  W_L("<menu");
+  /*--------------------------------------------------------------------------*/
+  /* Get Attributes                                                           */
+  /*--------------------------------------------------------------------------*/
+  for (attr = qs_get_attr(doc,node);
+       attr;
+       attr = qs_get_next_attr(doc,attr)) {
+    char *name   = qs_get_attr_name(doc,attr);
+    char *value  = qs_get_attr_value(doc,attr);
+    if (STRCASEEQ('t','T',"type",name)) {
+      if (value && (STRCASEEQ('d','D',"disc",value) || STRCASEEQ('c','C',"circle",value) || STRCASEEQ('s','S',"square",value))) {
+        W_L(" type=\"");
+        W_V(value);
+        W_L("\"");
+      }
+    }
+  }
+  W_L(">");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -2992,6 +3126,7 @@ s_xhtml_1_0_end_menu_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc *doc = xhtml->doc;
   W_L("</menu>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -3080,6 +3215,7 @@ s_xhtml_1_0_end_blink_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc = xhtml->doc;
   W_L("</blink>");
+  W_NLCODE();
   return xhtml->out;
 }
 
@@ -3128,6 +3264,13 @@ s_xhtml_1_0_start_marquee_tag(void *pdoc, Node *node)
         W_L("\"");
       }
     }
+    else if (STRCASEEQ('b','B',"bgcolor",name)) {
+      if (value && *value) {
+        W_L(" bgcolor=\"");
+        W_V(value);
+        W_L("\"");
+      }
+    }
   }
   W_L(">");
   return xhtml->out;
@@ -3148,6 +3291,7 @@ s_xhtml_1_0_end_marquee_tag(void *pdoc, Node *UNUSED(child))
   xhtml_t *xhtml = GET_XHTML(pdoc);
   Doc     *doc = xhtml->doc;
   W_L("</marquee>");
+  W_NLCODE();
   return xhtml->out;
 }
 /*
